var x$10 = "a";

var result$10 = `1 = ${x$10}`;

var x$$ = "b";

var result$$ = `2 = ${x$$}`;

var x$_ = "c";

var result$_ = `3 = ${x$_}`;

var x$Z = "d";

var result$Z = `4 = ${x$Z}`;

var x$Y = "e";

var result$Y = `5 = ${x$Y}`;

var x$X = "f";

var result$X = `6 = ${x$X}`;

var x$W = "g";

var result$W = `7 = ${x$W}`;

var x$V = "h";

var result$V = `8 = ${x$V}`;

var x$U = "i";

var result$U = `9 = ${x$U}`;

var x$T = "j";

var result$T = `10 = ${x$T}`;

var x$S = "k";

var result$S = `11 = ${x$S}`;

var x$R = "l";

var result$R = `12 = ${x$R}`;

var x$Q = "m";

var result$Q = `13 = ${x$Q}`;

var x$P = "n";

var result$P = `14 = ${x$P}`;

var x$O = "o";

var result$O = `15 = ${x$O}`;

var x$N = "p";

var result$N = `16 = ${x$N}`;

var x$M = "q";

var result$M = `17 = ${x$M}`;

var x$L = "r";

var result$L = `18 = ${x$L}`;

var x$K = "s";

var result$K = `19 = ${x$K}`;

var x$J = "t";

var result$J = `20 = ${x$J}`;

var x$I = "u";

var result$I = `21 = ${x$I}`;

var x$H = "v";

var result$H = `22 = ${x$H}`;

var x$G = "w";

var result$G = `23 = ${x$G}`;

var x$F = "x";

var result$F = `24 = ${x$F}`;

var x$E = "y";

var result$E = `25 = ${x$E}`;

var x$D = "z";

var result$D = `26 = ${x$D}`;

var x$C = "A";

var result$C = `27 = ${x$C}`;

var x$B = "B";

var result$B = `28 = ${x$B}`;

var x$A = "C";

var result$A = `29 = ${x$A}`;

var x$z = "D";

var result$z = `30 = ${x$z}`;

var x$y = "E";

var result$y = `31 = ${x$y}`;

var x$x = "F";

var result$x = `32 = ${x$x}`;

var x$w = "G";

var result$w = `33 = ${x$w}`;

var x$v = "H";

var result$v = `34 = ${x$v}`;

var x$u = "I";

var result$u = `35 = ${x$u}`;

var x$t = "J";

var result$t = `36 = ${x$t}`;

var x$s = "K";

var result$s = `37 = ${x$s}`;

var x$r = "L";

var result$r = `38 = ${x$r}`;

var x$q = "M";

var result$q = `39 = ${x$q}`;

var x$p = "N";

var result$p = `40 = ${x$p}`;

var x$o = "O";

var result$o = `41 = ${x$o}`;

var x$n = "P";

var result$n = `42 = ${x$n}`;

var x$m = "Q";

var result$m = `43 = ${x$m}`;

var x$l = "R";

var result$l = `44 = ${x$l}`;

var x$k = "S";

var result$k = `45 = ${x$k}`;

var x$j = "T";

var result$j = `46 = ${x$j}`;

var x$i = "U";

var result$i = `47 = ${x$i}`;

var x$h = "V";

var result$h = `48 = ${x$h}`;

var x$g = "W";

var result$g = `49 = ${x$g}`;

var x$f = "X";

var result$f = `50 = ${x$f}`;

var x$e = "Y";

var result$e = `51 = ${x$e}`;

var x$d = "Z";

var result$d = `52 = ${x$d}`;

var x$c = "1";

var result$c = `53 = ${x$c}`;

var x$b = "2";

var result$b = `54 = ${x$b}`;

var x$a = "3";

var result$a = `55 = ${x$a}`;

var x$9 = "4";

var result$9 = `56 = ${x$9}`;

var x$8 = "5";

var result$8 = `57 = ${x$8}`;

var x$7 = "6";

var result$7 = `58 = ${x$7}`;

var x$6 = "7";

var result$6 = `59 = ${x$6}`;

var x$5 = "8";

var result$5 = `60 = ${x$5}`;

var x$4 = "9";

var result$4 = `61 = ${x$4}`;

var x$3 = "0";

var result$3 = `62 = ${x$3}`;

var x$2 = "_";

var result$2 = `63 = ${x$2}`;

var x$1 = "$";

var result$1 = `64 = ${x$1}`;

var x = "0";

var result = `65 = ${x}`;

var results = [result$10,
result$$,
result$_,
result$Z,
result$Y,
result$X,
result$W,
result$V,
result$U,
result$T,
result$S,
result$R,
result$Q,
result$P,
result$O,
result$N,
result$M,
result$L,
result$K,
result$J,
result$I,
result$H,
result$G,
result$F,
result$E,
result$D,
result$C,
result$B,
result$A,
result$z,
result$y,
result$x,
result$w,
result$v,
result$u,
result$t,
result$s,
result$r,
result$q,
result$p,
result$o,
result$n,
result$m,
result$l,
result$k,
result$j,
result$i,
result$h,
result$g,
result$f,
result$e,
result$d,
result$c,
result$b,
result$a,
result$9,
result$8,
result$7,
result$6,
result$5,
result$4,
result$3,
result$2,
result$1,
result
];

console.log(results);
