import lib1, {value1} from './lib1.js';
console.log(lib1, value1);

import lib2, {value2} from './lib2.js';
console.log(lib2, value2);

import lib3, {value3} from './lib3.js';
console.log(lib3, value3);

import lib4, {value4} from './lib4.js';
console.log(lib4, value4);

import lib5, {value5} from './lib5.js';
console.log(lib5, value5);

export const value = 42;
export default 42;
