export { default as foo } from 'external-all';
export { default as bar } from 'external-default';
export { default as baz } from 'external-default-named';
export { default as quux } from 'external-default-namespace';
