module.exports = {
	description: 'imports default and named exports in same declaration'
};

// test copied from https://github.com/esnext/es6-module-transpiler/tree/master/test/examples/export-default-named-function
