'use strict';

var _virtualModule = require('./_virtual/_virtualModule-cjs-bbe1ad5e.mjs');
var _virtualWithExt = require('./_virtual/_virtualWithExt-cjs-b148ca33.mjs');
var _virtualWithAssetExt = require('./_virtual/_virtualWithAssetExt.str-cjs-0e561611.mjs');

assert.equal(_virtualModule.virtual, 'Virtual!');
assert.equal(_virtualWithExt.virtual2, 'Virtual2!');
assert.equal(_virtualWithAssetExt.virtual3, 'Virtual3!');
