var myBundle = (function (exports, external) {
	'use strict';



	Object.defineProperty(exports, 's', {
		enumerable: true,
		get: function () { return external.p; }
	});

	return exports;

})({}, external);
