System.register('bundle', [], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			const exports$1 = exports('exports', 1);
			const require = exports('require', 2);
			const module$1 = exports('module', 3);
			const __filename = exports('__filename', 4);
			const __dirname = exports('__dirname', 5);

		})
	};
}));
