define(['./custom_modules/@my-scope/my-base-pkg/index', './_virtual/index'], (function (index$1, index) { 'use strict';

	const base = index.__exports;

	var underBuild = {
		base
	};

	return underBuild;

}));
