'use strict';

var geometry = require('./generated-geometry.js');

var mod = /*#__PURE__*/Object.freeze({
	__proto__: null,
	geometry: geometry.geometry,
	volume: geometry.volume
});

console.log(mod);
