export { default as fooAuto, barAuto } from 'external-auto';
export { default as fooDefault, barDefault } from 'external-default';
export { default as fooDefaultOnly } from 'external-defaultOnly';
export { default as fooEsModule, barEsModule } from 'external-esModule';
export * as externalAuto from 'external-auto';
export * as externalDefault from 'external-default';
export * as externalEsModule from 'external-esModule';
