import { virtual } from './_virtual/_virtualModule-es-22303f19.mjs';
import { virtual2 } from './_virtual/_virtualWithExt-es-6330be37.mjs';
import { virtual3 } from './_virtual/_virtualWithAssetExt.str-es-07beb78c.mjs';

assert.equal(virtual, 'Virtual!');
assert.equal(virtual2, 'Virtual2!');
assert.equal(virtual3, 'Virtual3!');
