var x;
var y = undefined;
var z;

if (x) console.log('removed');
else console.log('retained');

if (y) console.log('removed');
else console.log('retained');

if (z) console.log('retained');
else console.log('retained');

z = 1;
