var name$b = 5;

var name$a = 5;

var name$9 = 5;

var name$8 = 5;

var name$7 = 5;

var name$6 = 5;

var name$5 = 5;

var name$4 = 5;

var name$3 = 5;

var name$2 = 5;

var name$1 = 5;

console.log(name$b);
console.log(name$a);
console.log(name$9);
console.log(name$8);
console.log(name$7);
console.log(name$6);
console.log(name$5);
console.log(name$4);
console.log(name$3);
console.log(name$2);
console.log(name$1);

var name = 'name';

export { name };
