define(['exports'], (function (exports) { 'use strict';

	const shared1 = 'shared1';
	const foo$1 = 'foo1';

	var shared2 = 'shared2';
	const foo = 'foo2';

	exports.a = shared2;
	exports.b = foo;
	exports.f = foo$1;
	exports.s = shared1;

}));
