'use strict';

var external = require('external');

const renamedIndirectOverride = external.indirectOverride;

Object.defineProperty(exports, 'renamedDirectOverride', {
	enumerable: true,
	get: function () { return external.directOverride; }
});
exports.renamedIndirectOverride = renamedIndirectOverride;
Object.keys(external).forEach(function (k) {
	if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
		enumerable: true,
		get: function () { return external[k]; }
	});
});
