import { missing1 as _missingExportShim } from './dep1.js';
import { missing2 as _missingExportShim$1, previousShimmedExport as _missingExportShim$2 } from './dep2.js';

console.log(_missingExportShim, _missingExportShim$1, _missingExportShim$2);
