define(['exports'], (function (exports) { 'use strict';

	const small3 = '12345678901234567890';

	const small2 = '12345678901234567890';

	const small1 = '12345678901234567890';

	exports.small1 = small1;
	exports.small2 = small2;
	exports.small3 = small3;

}));
