(function (factory) {
	typeof define === 'function' && define.amd ? define(factory) :
	factory();
})((function () { 'use strict';

	import('external');
	import(globalThis.unknown);
	import('resolvedString');
	import('resolved-id');
	import('resolved-different');
	import('unresolved');

}));
