const small1 = 'small1';

var small1$1 = /*#__PURE__*/Object.freeze({
	__proto__: null,
	small1: small1
});

const small2 = 'small2';

var small2$1 = /*#__PURE__*/Object.freeze({
	__proto__: null,
	small2: small2
});

export { small2$1 as a, small1$1 as s };
