const foo = 'bar';

export { foo as ø };
export { foo as z };
export { foo as ZZZ };
export { foo as $ };
export { foo as ς };
export default foo;
export { foo as ___ };
export { foo as aa };
export { foo as AA };
export * as namespace from './bar.js';
export { foo as Z };
export { foo as ö };
export { foo as A };
export { foo };
export { foo as $$$ };
