System.register('myBundle', [], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			const localIsNaN = isNaN;

			const isNaN$1 = exports('isNaN', localIsNaN);

		})
	};
}));
