define(['exports', './foo-amd-0f9dc16c', './nested/bar-amd-f038b10c', './nested/baz-amd-d3de4cc0', './no-ext-amd-9d2c6ef6'], (function (exports, foo, bar, baz, noExt) { 'use strict';



	exports.foo = foo;
	exports.bar = bar;
	exports.baz = baz;
	exports.noExt = noExt;

}));
