import require$$0 from 'external';
import './other.js';
import { __exports as other } from './_virtual/other.js';

const external = require$$0;
const { value } = other;

console.log(external, value);

var commonjs = 42;

export { commonjs as default };
