define(['./generated-chunk-c', './generated-chunk-b'], (function (chunkC, chunkB) { 'use strict';

	console.log('dep1');

	console.log('dep-a');

	console.log('dep2-a');

	console.log('dep3-a');

	console.log('main-a');

}));
