define(['exports'], (function (exports) { 'use strict';

	const small1 = 'small1';

	var small1$1 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		small1: small1
	});

	const small2 = 'small2';

	var small2$1 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		small2: small2
	});

	exports.small1 = small1$1;
	exports.small2 = small2$1;

}));
