var bundle = (function (exports, external) {
	'use strict';

	const value = 42;

	exports.default = external;
	exports.value = value;

	Object.defineProperty(exports, '__esModule', { value: true });

	return exports;

})({}, external);
