(function (relative, absolute, relativeExtension, absoluteExtension) {
	'use strict';

	console.log(relative, absolute, relativeExtension, absoluteExtension);

})(relative, absolute, relativeExtension, absoluteExtension);
