'use strict';

var relative = require('./relative');
var absolute = require('abso/lute');
var relativeExtension = require('./relative.js');
var absoluteExtension = require('abso/lute.js');

console.log(relative, absolute, relativeExtension, absoluteExtension);
