import { foo } from 'external-all';
import { bar } from 'external-named';
import { baz } from 'external-default-named';
import { quux } from 'external-named-namespace';
console.log(foo, bar, baz, quux);
