System.register([], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			var separate = exports('default', 'separate');
			const x = exports('x', 2);
			console.log('separate');

		})
	};
}));
