System.register([], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			const value2 = exports('v', 'all');

			const value3 = exports('a', 'main1 and main2');

		})
	};
}));
