import asset from './logo.svg';
import asset2 from './logo2.svg';

if (false) {
	const image = document.createElement('img');
	image.src = asset;
	document.body.appendChild(image);
}
if (true) {
	const image = document.createElement('img');
	image.src = asset2;
	document.body.appendChild(image);
}
