import '../custom_modules/@my-scope/my-base-pkg/index.js';
import { __exports as myBasePkg } from '../_virtual/index.js';

const base2 = myBasePkg;

var module = {
  base2,
};

export { module as default };
