'use strict';

Promise.resolve().then(function () { return require('./generated-dynamic2.js'); });
Promise.resolve().then(function () { return require('./generated-dynamic1.js'); });
console.log('main');
