define(['./generated-geometry'], (function (geometry) { 'use strict';

	var mod = /*#__PURE__*/Object.freeze({
		__proto__: null,
		geometry: geometry.geometry,
		volume: geometry.volume
	});

	console.log(mod);

}));
