System.register([], (function (exports, module) {
	'use strict';
	return {
		execute: (function () {

			const shared = exports('shared', 'shared');

			module.import('./generated-dynamic1.js');

		})
	};
}));
