var myBundle = (function (exports, x) {
	'use strict';

	function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

	var x__default = /*#__PURE__*/_interopDefaultCompat(x);



	Object.defineProperty(exports, 'x', {
		enumerable: true,
		get: function () { return x__default.default; }
	});

	return exports;

})({}, x);
