'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _missingExportShim = void 0;

function x (arg) {
  sideEffect(arg);
}

exports.default = _missingExportShim;
exports.missingExport = _missingExportShim;
exports.missingFn = _missingExportShim;
exports.x = x;
