
// banner-pre-2
// banner-string-pre-2
// banner-pre-5
// banner-string-pre-5
// banner-null-1
// banner-string-null-1
// banner-undefined-6
// banner-string-undefined-6
// banner-post-3
// banner-string-post-3
// banner-post-4
// banner-string-post-4
// intro-pre-2

// intro-string-pre-2

// intro-pre-5

// intro-string-pre-5

// intro-null-1

// intro-string-null-1

// intro-undefined-6

// intro-string-undefined-6

// intro-post-3

// intro-string-post-3

// intro-post-4

// intro-string-post-4

console.log('main');



// outro-pre-2

// outro-string-pre-2

// outro-pre-5

// outro-string-pre-5

// outro-null-1

// outro-string-null-1

// outro-undefined-6

// outro-string-undefined-6

// outro-post-3

// outro-string-post-3

// outro-post-4

// outro-string-post-4

// footer-pre-2
// footer-string-pre-2
// footer-pre-5
// footer-string-pre-5
// footer-null-1
// footer-string-null-1
// footer-undefined-6
// footer-string-undefined-6
// footer-post-3
// footer-string-post-3
// footer-post-4
// footer-string-post-4
