export function esmTest() {
	return {
		transform(code, id) {
			if (id.includes("\0")) return null;
			const name = JSON.stringify(id.replace(/^.*\/advanced-esm\//, "esm-test: "));
			return `console.log(${name});\n${code}`;
		}
	};
}
