'use strict';

/* this is an intro */

// intro 1

// intro 2

// intro 3

// intro 4

var a = require('external');

console.log( a );
console.log( a.b );

var main = 42;

module.exports = main;

/* this is an outro */

// outro 1

// outro 2

// outro 3

// outro 4
