/* banner-option amd.js */
/* banner-first amd.js */
/* banner-second amd.js */
define((function () { 'use strict';

	/* intro-option amd.js */

	/* intro-first amd.js */

	/* intro-second amd.js */

	console.log('main');

	/* outro-option amd.js */

	/* outro-first amd.js */

	/* outro-second amd.js */

}));
/* footer-option amd.js */
/* footer-first amd.js */
/* footer-second amd.js */
