define(['external'], (function (a) { 'use strict';

	/* this is an intro */

	// intro 1

	// intro 2

	// intro 3

	// intro 4

	console.log( a );
	console.log( a.b );

	var main = 42;

	return main;

	/* this is an outro */

	// outro 1

	// outro 2

	// outro 3

	// outro 4

}));
