System.register(['./generated-manual-outer.js', './generated-manual-middle.js', './generated-manual-inner.js'], (function () {
	'use strict';
	return {
		setters: [null, null, null],
		execute: (function () {

			console.log('main');

		})
	};
}));
