(function (factory) {
	typeof define === 'function' && define.amd ? define(factory) :
	factory();
})((function () { 'use strict';

	var dep = "default";

	var dep$1 = /*#__PURE__*/Object.freeze({
		__proto__: null,
		default: dep
	});

	console.log(dep$1);

}));
