System.register([], (function (exports) {
	'use strict';
	return {
		execute: (function () {

			const small1 = 'small1';

			var small1$1 = /*#__PURE__*/Object.freeze({
				__proto__: null,
				small1: small1
			});
			exports('s', small1$1);

			const small2 = 'small2';

			var small2$1 = /*#__PURE__*/Object.freeze({
				__proto__: null,
				small2: small2
			});
			exports('a', small2$1);

		})
	};
}));
