var a, b;
console.log(a, b);

var c, d;
console.log(c, d);

const e = 1, f = 2;
console.log(e, f);

const g = 3, h = 4;
console.log(g, h);

var i, j;

var k, l;

const m = 1, n = 2;

const o = 3, p = 4;

export { i, j, k, l, m, n, o, p };
