define(['./_virtual/_virtualModule-amd-03552e5c.mjs', './_virtual/_virtualWithExt-amd-e9dc1b7e.mjs', './_virtual/_virtualWithAssetExt.str-amd-724c602d.mjs'], (function (_virtualModule, _virtualWithExt, _virtualWithAssetExt) { 'use strict';

	assert.equal(_virtualModule.virtual, 'Virtual!');
	assert.equal(_virtualWithExt.virtual2, 'Virtual2!');
	assert.equal(_virtualWithAssetExt.virtual3, 'Virtual3!');

}));
