'use strict';

var _1 = require('./1.js');
var _2 = require('./2.js');
var _3 = require('./3.js');
var _4 = require('./4.js');
var _5 = require('./5.js');



exports.x1 = _1.x1;
exports.x2 = _2.x2;
exports.x3 = _3.x3;
exports.x4 = _4.x4;
exports.x5 = _5.x5;
