/* banner-option iife.js */
/* banner-first iife.js */
/* banner-second iife.js */
(function () {
	'use strict';

	/* intro-option iife.js */

	/* intro-first iife.js */

	/* intro-second iife.js */

	console.log('main');

	/* outro-option iife.js */

	/* outro-first iife.js */

	/* outro-second iife.js */

})();
/* footer-option iife.js */
/* footer-first iife.js */
/* footer-second iife.js */
