System.register([], (function (exports, module) {
	'use strict';
	return {
		execute: (function () {

			var asset1 = new URL('assets/asset-8e3dd2ea', module.meta.url).href;

			var asset2 = new URL('assets/asset-75590fc1', module.meta.url).href;

			var asset99 = new URL('assets/asset-60cc5dc9', module.meta.url).href;

			console.log(asset1, asset2, asset99);

		})
	};
}));
