'use strict';

var answer = require('./answer.num.js');
var lorem = require('./lorem.str.js');
var noExt = require('./no-ext.js');



exports.answer = answer;
exports.lorem = lorem;
exports.noExt = noExt;
